from typing import Any


class ErrorMixin:
    """The mixin for base exceptions.

    :param Any ctx: content variables.
    """

    #: Type identification code.
    code: str
    #: Error message template.
    msg_template: str

    def __init__(self, **ctx: Any) -> None:
        self.__dict__ = ctx

    def __str__(self) -> str:
        return self.msg_template.format(**self.__dict__)


class Error(ErrorMixin, Exception):
    """Base-class for all exceptions raised by this plugin."""


class PathError(Error):
    """The base-class for all path-related exceptions.

    :param str path: path string.
    """

    def __init__(self, path: str, **kwargs: Any) -> None:
        super(PathError, self).__init__(path=path)


class PathNotAExecutableError(PathError):
    code = "file.not_a_executable"
    msg_template = 'path "{path}" does not point to a executable file'


class LocatorError(Error):
    """The base-class for all locator-related exceptions.

    :param str path: locatore string.
    """

    def __init__(self, locator: str, **kwargs: Any) -> None:
        super(LocatorError, self).__init__(locator=locator)


class LocatorNotAValidSelectorOrXPathError(LocatorError):
    code = "locator.not_a_valid_selector_or_xpath"
    msg_template = 'locator "{locator}" is not a valid selector or xpath string.'


class ElementError(Error):
    """The base-class for all element-related exceptions.

    :param str locator: element locator string.
    """

    def __init__(self, locator: str, **kwargs: Any) -> None:
        super(ElementError, self).__init__(locator=locator, **kwargs)


class ElementNotExistError(ElementError):
    code = "element.not_exist"
    msg_template = 'Element "{locator}" not exist.'


class ElementTimeoutError(ElementError):
    code = "element.timeout"
    msg_template = (
        'Wait for element "{locator}" to {action} failed: timeout {timeout}ms exceeds.'
    )

    def __init__(self, locator: str, timeout: int, action: str = "appear") -> None:
        super(ElementTimeoutError, self).__init__(
            locator=locator, timeout=timeout, action=action
        )



import sys

import cssselect
from lxml import etree

from pytest_pyppeteer.errors import LocatorNotAValidSelectorOrXPathError

CHROME_EXECUTABLE = {
    "mac": r"/Applications/Google Chrome.app/Contents/MacOS/Google Chrome",
    "win64": r"C:\Program Files\Google\Chrome\Application\chrome.exe",
    "win32": r"C:\Program Files (x86)\Google\Chrome\Application\chrome.exe",
}


def current_platform() -> str:
    """Return current platform name.

    :return: the platform name. only be one of ``mac`` ``win64`` ``win32``.
    :raise OSError: if current platform is not supported.
    """
    if sys.platform.startswith("darwin"):
        return "mac"
    if sys.platform.startswith("win"):
        if sys.maxsize > 2 ** 31 - 1:
            return "win64"
        return "win32"
    raise OSError(
        'Unsupport platform: "{}"; Now only support "win64", "win32", "mac"'.format(
            sys.platform
        )
    )


def parse_locator(locator: str) -> tuple:
    """validate that the locator string must a valid css or
    xpath.

    :param locator: a locator string.
    :return: a tuple contains locator_type and locator_string.
    :raise LocatorNotAValidSelectorOrXPath: locator is not a
           valid selector or xpath string.
    """
    if not isinstance(locator, str):
        raise TypeError("Locator not a string.")
    # CSS selector
    try:
        cssselect.parse(locator)
    except cssselect.SelectorSyntaxError:
        pass
    else:
        return "css", locator
    # XPath
    try:
        etree.XPath(locator)
    except etree.XPathSyntaxError:
        pass
    else:
        return "xpath", locator
    raise LocatorNotAValidSelectorOrXPathError(locator=locator)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from _pytest.nodes import Item


async def pytest_pyppeteer_runtest_makereport_call_debug(item: "Item") -> None:
    """Called to add debug information when each of the call(not
    setup/teardown) runtest phases of a test failed item. e.g.
    save screenshot.

    :param Item item: the pytest item object.
    :return: None
    """



import asyncio
import inspect
import logging
from typing import TYPE_CHECKING, Any, Awaitable, Callable, List, Optional

import pytest
from pyppeteer import launch

from pytest_pyppeteer.models import Browser, Options
from pytest_pyppeteer.utils import CHROME_EXECUTABLE, current_platform

if TYPE_CHECKING:
    from _pytest.config import Config, PytestPluginManager
    from _pytest.config.argparsing import Parser
    from _pytest.fixtures import FixtureRequest
    from _pytest.nodes import Item
    from _pytest.runner import TestReport

LOGGER = logging.getLogger(__name__)


def pytest_addhooks(pluginmanager: "PytestPluginManager") -> None:
    """Add new hooks.

    :param _pytest.config.PytestPluginManager pluginmanager:
    :return: ``None``
    """
    from . import hooks

    pluginmanager.add_hookspecs(hooks)


def pytest_configure(config: "Config") -> None:
    """Perform initial configuration as follows:

    * Add ``pytest.mark.option`` marker to the ini-file option.

    :param _pyest.config.Config config: pytest config object.
    :return: None

    .. note::

        This is a ``pytest hook function`` which is called for
        every plugin and initial conftest file after command
        line options have been parsed. After that, the hook
        is called for other conftest files as they are imported.
    """
    config.addinivalue_line(
        "markers",
        "options(kwargs): add or change existing options. "
        "specify options as keyword argument, e.g. "
        "@pytest.mark.options(devtools=True)",
    )


@pytest.mark.tryfirst
def pytest_collection_modifyitems(items: List["Item"]) -> None:
    """Modify the items collected by pytest as follows:

    * Because all test items using pyppeteer should be an
      asyncio coroutine, here the `pytest.mark.asyncio`_
      marker is automatically added to each test item
      collected by pytest.

    :param List[pytest.Item] items: the list of items objects
           collected by pytest.
    :return: None

    .. note::

        This is a ``pytest hook function`` which is called
        after collection of all test items is completed.

    ..  _pytest.mark.asyncio:
        https://github.com/pytest-dev/pytest-asyncio#pytestmarkasyncio
    """
    items[:] = [add_asyncio_marker(item) for item in items]


def add_asyncio_marker(item: "Item") -> "Item":
    """Add `pytest.mark.asyncio`_ marker to the specified item.

    If the marker is already exists, return the item directly.

    :param pytest.Item item: the pytest item object.
    :return: the marked item object.

    ..  _pytest.mark.asyncio:
        https://github.com/pytest-dev/pytest-asyncio#pytestmarkasyncio
    """
    if "asyncio" not in item.keywords and is_coroutine(item.obj):
        item.add_marker(pytest.mark.asyncio)
    return item


def is_coroutine(obj: Any) -> bool:
    """Check to see if an object is really an asyncio coroutine.

    :param Any obj: any object.
    :return: `True` or `False`.
    """
    return asyncio.iscoroutinefunction(obj) or inspect.isgeneratorfunction(obj)


@pytest.hookimpl(hookwrapper=True, tryfirst=True)
def pytest_runtest_makereport(item: "Item") -> None:
    """Implement this pytest hook in wrapper mode, the added
    behaviors as follows:

    * Register a new
      :py:func:`hooks.pytest_pyppeteer_runtest_makereport_call_debug`
      hook which called when a actual failing test calls not setup/teardown.

    :param pytest.Item item: the pytest item object.
    :return: None
    """
    # execute all other hooks to obtain the report object
    outcome = yield
    res: TestReport = outcome.get_result()
    setattr(item, f"res_{res.when}", res)

    # we only deal with actual failing test calls, not setup/teardown
    if res.when == "call" and res.failed:
        hook: List[
            Awaitable
        ] = item.ihook.pytest_pyppeteer_runtest_makereport_call_debug(item=item)
        if hook:
            loop = asyncio.get_event_loop()
            loop.run_until_complete(hook[0])


def pytest_addoption(parser: "Parser") -> None:
    """Register new command line arguments and ini-file values.

    Create a new command line option group named `pyppeteer`, and add the
    following new options in it:

    * ``--executable-path``: path to a Chromium or Chrome executable.

    * ``--headless``: run browser in headless mode.

    * ``--args``: additional args to pass to the browser instance. more
      details refer to :py:func:`args` fixture.

    * ``--window-size``: set the initial browser window size. Defaults
      to 800 * 600. ``--window-size 0 0`` means to starts the browser
      maximized.

    :param _pytest.config.argparsing.Parser parser: parser for command
           line arguments and ini-file values.
    :return: None

    .. note::

        This is a ``pytest hook function`` which be called once at the beginning
        of a test run to register argparse-style options and ini-style config values.

        There are two ways to register new options, respectively:

        * To register a command line option, call
          :py:meth:`parser.addoption(...) <pytest:_pytest.config.argparsing.Parser.addoption>`.

        * To register an ini-file option, call
          :py:meth:`parser.addini(...) <pytest:_pytest.config.argparsing.Parser.addini>`.

        And the options can later be accessed through the
        :py:class:`Config <pytest:_pytest.config.Config>` object, respectively:

        * To retrieve the value of a command line option, call
          :py:meth:`config.getoption(name) <pytest:_pytest.config.Config.getoption>`.

        * To retrieve a value read from an ini-style file, call
          :py:meth:`config.getini(name) <pytest:_pytest.config.Config.getini>`.

        The ``Config`` object is passed around on many pytest internal objects via the
        ``.config`` attribute or can be retrieved as the
        :py:func:`pytestconfig <pytest:_pytest.fixtures.pytestconfig>` fixture.
    """
    # Create a option group named "pyppeteer"
    group = parser.getgroup("pyppeteer", description="pyppeteer")
    group.addoption(
        "--executable-path", help="path to a Chromium or Chrome executable."
    )

    group.addoption(
        "--headless", action="store_true", help="run browser in headless mode."
    )

    group.addoption(
        "--args",
        action="append",
        nargs="+",
        help="additional args to pass to the browser instance.",
    )

    group.addoption(
        "--window-size",
        nargs=2,
        default=["800", "600"],
        help="set the initial browser window size.",
    )


@pytest.fixture(scope="session")
def executable_path(pytestconfig: "Config") -> Optional[str]:
    """``Session-scoped fixture`` that return Chrome or Chromium executable path.

    The fixture behaviors follow this procedure:

    1. Return the value passed in from command line option of `--executable-path`,
       if it's not ``None``.

    2. Return the default installation location of Chrome in current platform,
       but now only support ``win64``, ``win32`` and ``mac`` platform.

       For other platforms, pyppeteer will downloads the recent version of Chromium
       when called first time. If you don't prefer this behavior, you can specify
       an exact path by overwrite this fixture:

       Example::

           @pytest.fixture(scope="session")
           def executable_path(executable_path):
               if executable_path is None:
                   return "path/to/Chrome/or/Chromium"
                   return executable_path


    :param _pytest.config.Config pytestconfig: a session-scoped fixture that return
           config object.
    :return: return Chrome or Chromium executable path string. but if current platform
             isn't supported, return ``None``.
    """
    path = pytestconfig.getoption("--executable-path")
    if path:
        return path

    try:
        return CHROME_EXECUTABLE[current_platform()]
    except OSError as e:
        # Unsupported platform
        LOGGER.error(e)
        return None


@pytest.fixture(scope="session")
def args(pytestconfig: "Config") -> List[str]:
    """``Session-scoped fixture`` that return a list of additional
    args in the `List of Chromium Command Line Arguments`_ to pass
    to the browser instance.

    You can use it by command-line option:

    Example::

        $ pytest --args proxy-server "localhost:5555,direct://" --args proxy-bypass-list "192.0.0.1/8;10.0.0.1/8"

    Or overwrite it in your test:

    Example::

        @pytest.fixture(scope="session")
        def args(args) -> List[str]:
            return args + [
                "--proxy-server=localhost:5555,direct://",
                "--proxy-bypass-list=192.0.0.1/8;10.0.0.1/8",
            ]

    :param _pytest.config.Config pytestconfig: a session-scoped
           fixture that return config object.
    :return: a list of arguments string. return ``list()`` if no
             ``--args`` passed in the command-line.

    .. _List of Chromium Command Line Arguments:
       https://peter.sh/experiments/chromium-command-line-switches/
    """
    args: list = pytestconfig.getoption("--args")
    if args:
        return ["--" + "=".join(arg_list) for arg_list in args]
    else:
        return list()


@pytest.fixture(scope="session")
def session_options(
    pytestconfig: "Config", args: List[str], executable_path: str
) -> "Options":
    """``Session-scoped fixture`` that return a
    :py:class:`models.Options` object used to initialize browser.

    :param _pytest.config.Config pytestconfig: a session-scoped
           fixture that return config object.
    :param List[str] args: a session-scoped fixture that return
           a list of additional args to pass to the browser instance.
    :param str executable_path: a session-scoped fixture that return
           Chrome or Chromium executable path.
    :return: a :py:class:`models.Options` object used to initialize
             browser.
    """
    headless: bool = pytestconfig.getoption("--headless")
    width, hight = pytestconfig.getoption("--window-size")
    if width == "0" and hight == "0":
        if current_platform() == "mac":
            args.append("--start-fullscreen")
        else:
            args.append("--start-maximized")
    else:
        args.append("--window-size={},{}".format(width, hight))
    return Options(args=args, executablePath=executable_path, headless=headless)


@pytest.fixture
def options(request: "FixtureRequest", session_options: "Options") -> "Options":
    """``Function-scoped fixture`` that return a
    :py:class:`models.Options` object used to initialize browser.

    This fixture contains all of :py:func:`session_options`, plus
    any options specified by the options markers. Any change to
    these options will apply only to the tests covered by scope
    of the fixture override.

    Example::

        @pytest.mark.options(devtools=True)
        async def test_options_mark(pyppeteer):
            ...

    :param _pytest.fixture.FixtureRequest request: A request
           object gives access to the requesting test context.
    :param Options session_options: a :py:class:`models.Options`
           object from :py:func:`session_options`.
    :return: a :py:class:`models.Options` object used to initialize
             browser.
    """
    return session_options.copy(update=get_options_from_markers(request.node))


def get_options_from_markers(item: "Item") -> dict:
    """Get the options from the ``options`` markers of test item.
    And there are only apply on the current test item.

    :param Item item: the test item object.
    :return: an dict contains options.
    """
    options_dict = dict()
    for node, mark in item.iter_markers_with_node("options"):
        LOGGER.debug(
            "{0} marker <{1.name}> "
            "contains kwargs <{1.kwargs}>".format(node.__class__.__name__, mark)
        )
        options_dict.update(mark.kwargs)
    LOGGER.info("Options from markers: {}".format(options_dict))
    return options_dict


@pytest.fixture
async def pyppeteer_factory(options: "Options") -> Callable:
    """``Function-scoped fixture`` that return a pyppeteer
    browser factory.

    :param Options options: a :py:class:`models.Options`
           object used to initialize browser
    :yield: a pyppeteer browser factory.
    """
    browsers: List[Browser] = list()

    async def _factory() -> "Browser":
        LOGGER.info("Options to initialize browser instance: {}".format(options))
        b = Browser(pyppeteer_browser=await launch(options=options.dict()))
        browsers.append(b)
        return b

    yield _factory

    for browser in browsers:
        await browser.close()


@pytest.fixture
async def pyppeteer(pyppeteer_factory: "Callable") -> "Browser":
    """``Function-scoped fixture`` that return a pyppeteer
    browser instance.

    :param Callable pyppeteer_factory: pyppeteer factory
    :yield: a pyppeteer browser instance.
    """
    yield await pyppeteer_factory()



import asyncio
import logging
import os
from typing import TYPE_CHECKING, List, Optional, Union

from pydantic import BaseModel, validator
from pyppeteer.browser import Browser as PyppeteerBrowser
from pyppeteer.errors import ElementHandleError, TimeoutError
from pyppeteer.page import Page as PyppeteerPage

from pytest_pyppeteer.errors import (
    ElementNotExistError,
    ElementTimeoutError,
    PathNotAExecutableError,
)
from pytest_pyppeteer.utils import parse_locator

if TYPE_CHECKING:
    from pyppeteer.element_handle import ElementHandle

LOGGER = logging.getLogger(__name__)


class ViewPort(BaseModel):
    """Keep the consistency of each page's viewport in a browser instance.

    One of the most important use is as the standard setting model for
    :py:class:`Options.defaultViewport`.
    """

    #: Page width in pixels. Defaults to ``800``.
    width: int = 800

    #: Page height in pixels. Defaults to ``600``.
    height: int = 600

    #: Specify device scale factor (can be thought of as dpr).
    #: Defaults to ``1.0``.
    deviceScaleFactor: float = 1.0

    #: Whether the ``meta viewport`` tag is taken into account.
    #: Defaults to ``False``.
    isMobile: bool = False

    #: Specifies if viewport supports touch events. Defaults to ``False``.
    hasTouch: bool = False

    #: Specifies if viewport is in landscape mode. Defaults to ``False``.
    isLandscape: bool = False


class Options(BaseModel):
    """The standard setting model for pyppeteer launcher."""

    #: Additional arguments to pass to the browser instance. The list
    #: of Chromium flags can be found
    #: `here <https://peter.sh/experiments/chromium-command-line-switches/>`_.
    #: Defaults to ``list()``.
    args: List[str] = list()

    #: Automatically close browser process when script completed.
    #: Defaults to ``True``.
    autoClose: bool = True

    #: Set a consistent viewport for each page. Defaults to a default
    #: :py:class:`ViewPort` instance. ``None`` means disables the
    #: default viewport.
    defaultViewport: Optional["ViewPort"] = ViewPort()

    #: Whether to auto-open a DevTools panel for each tab. If this
    #: option is ``True``, the ``headless`` option will be set ``False``.
    #: Defaults to ``False``.
    devtools: bool = False

    #: Whether to pipe the browser process stdout and stderr into
    #: ``process.stdout`` and ``process.stderr``. Defaults to ``False``.
    dumpio: bool = False

    #: Specify environment variables that will be visible to the
    #: browser. ``None`` means that same as python process.
    #: Defaults to ``None``.
    env: Optional[dict] = None

    #: Path to a Chromium or Chrome executable. ``None`` means use the
    #: default bundled Chromium. Defaults to ``None``.
    executablePath: Optional[str] = None

    #: Close the browser process on `Ctrl-C`. Defaults to ``True``.
    handleSIGINT: bool = True

    #: Close the browser process on `SIGTERM`. Defaults to ``True``.
    handleSIGTERM: bool = True

    #: Close the browser process on `SIGHUP`. Defaults to ``True``.
    handleSIGHUP: bool = True

    #: Whether to run browser in headless mode. Defaults to ``False``.
    headless: bool = False

    #: Whether to ignore HTTPS errors. Defaults to ``True``.
    ignoreHTTPSErrors: bool = True

    #: If ``True``, then do not use pyppeteer's default args. If a
    #: list is given, then filter out the given default args.
    #: Dangerous option; use with care. Defaults to ``False``.
    ignoreDefaultArgs: Union[bool, List[str]] = False

    #: Log level to print logs. ``None`` means that same as the root logger.
    #: Defaults to ``None``.
    logLevel: Union[int, str, None] = None

    #: Slow down operations by the specified amount of milliseconds.
    #: useful so that you can see what is going on. Defaults to ``0.0``.
    slowMo: float = 0.0

    #: Path to a
    #: `User Data Directory <https://chromium.googlesource.com/chromium/src/+/master/docs/user_data_dir.md>`_.
    #: Defaults to ``None``.
    userDataDir: Optional[str] = None

    class Config:
        """Control the behaviours of pydantic model."""

        #: whether to allow arbitrary user types for fields (they are
        #: validated simply by checking if the value is an instance
        #: of the type). If False, RuntimeError will be raised on model
        #: declaration.
        arbitrary_types_allowed = True

    @validator("executablePath", pre=True)
    def validate_executable_path(cls, path: Optional[str]) -> Optional[str]:
        """Validate that the specified ``executablePath`` must point
        to an executable.

        :param str path: path string.
        :return: path string.
        :raise PathNotAExecutableError: if path does not point to a
               executable file.
        """
        if path and not (os.path.isfile(path) and os.access(path, os.X_OK)):
            raise PathNotAExecutableError(path=os.path.abspath(path))
        return path


class Browser(BaseModel):
    #: a pyppeteer browser object.
    pyppeteer_browser: "PyppeteerBrowser"

    class Config:
        """Control the behaviours of pydantic model."""

        #: whether to allow arbitrary user types for fields (they are
        #: validated simply by checking if the value is an instance
        #: of the type). If False, RuntimeError will be raised on model
        #: declaration.
        arbitrary_types_allowed = True

    def __getattr__(self, name):
        return getattr(self.pyppeteer_browser, name)

    async def new_page(self) -> "Page":
        """Make new page on this browser and return its object.

        :return: a :py:class:`Page` object.
        """
        pyppeteer_page = await self.pyppeteer_browser.newPage()
        # Make sure page and browser size as same.
        dimensions = await pyppeteer_page.evaluate(
            """() => {
            return {
                width: window.outerWidth,
                height: window.outerHeight,
                deviceScaleFactor: window.devicePixelRatio,
            }
        }"""
        )
        LOGGER.info('Resize page viewport to "{}"'.format(dimensions))
        await pyppeteer_page.setViewport(dimensions)
        return Page(pyppeteer_page=pyppeteer_page)


async def _clickable(element: "ElementHandle") -> None:
    for i in range(10):
        try:
            await element._clickablePoint()
        except ElementHandleError as e:
            LOGGER.error(e)
            LOGGER.info("Element not clickable. waiting... and try again.")
            await asyncio.sleep(0.5)
        else:
            break


class Page(BaseModel):
    #: a pyppeteer page object.
    pyppeteer_page: "PyppeteerPage"

    class Config:
        """Control the behaviours of pydantic model."""

        #: whether to allow arbitrary user types for fields (they are
        #: validated simply by checking if the value is an instance
        #: of the type). If False, RuntimeError will be raised on model
        #: declaration.
        arbitrary_types_allowed = True

    def __getattr__(self, name):
        return getattr(self.pyppeteer_page, name)

    async def query_locator(self, locator: str) -> Optional["ElementHandle"]:
        """Get the element which match ``locator``.

        If no element matches the ``locator``, return ``None``.

        :param str locator: a selector or xpath string
        :return: an element handle or ``None``.
        """
        _type, locator_string = parse_locator(locator)
        if _type == "css":
            return await self.pyppeteer_page.querySelector(locator_string)
        elif _type == "xpath":
            element_list = await self.pyppeteer_page.xpath(locator_string)
            return element_list[0] if element_list else None

    async def waitfor(
        self,
        locator: str,
        visible: bool = True,
        hidden: bool = False,
        timeout: int = 30000,
    ) -> None:
        """Wait until element which matches ``locator``.

        :param str locator: a selector or xpath string.
        :param bool visible: Wait for element to be present in DOM and to be
               visible; i.e. to not have ``display: none`` or ``visibility: hidden``
               CSS properties. Defaults to ``True``.
        :param bool hidden: Wait for element to not be found in the DOM or to
               be hidden, i.e. have ``display: none`` or ``visibility: hidden`` CSS
               properties. Defaults to ``False``.
        :param int timeout: Maximum time to wait for in milliseconds.
               Defaults to 30000 (30 seconds). Pass ``0`` to disable timeout.
        :return: None
        :raise ElementTimeoutError: Timeout exceeded while wait for ``locator``.
        """
        _type, locator_string = parse_locator(locator)
        options = {"visible": visible, "hidden": hidden, "timeout": timeout}
        try:
            if _type == "css":
                await self.pyppeteer_page.waitForSelector(
                    locator_string, options=options
                )
            elif _type == "xpath":
                await self.pyppeteer_page.waitForXPath(locator_string, options=options)
        except TimeoutError as e:
            action = "disappear" if hidden else "appear"
            raise ElementTimeoutError(
                locator=locator, timeout=timeout, action=action
            ) from e

    async def type(self, locator: str, text: str, delay: int = 0, clear: bool = False):
        """Focus the element which matches ``locator`` and then
        type text.

        :param locator: a selector or xpath string.
        :param text: what you want to type into.
        :param int delay: specifies time to wait between key presses
               in milliseconds. Defaults to 0.
        :param bool clear: whether to clear existing content befor
               typing. Defaults to ``False``.
        :return:
        """
        element = await self.query_locator(locator)
        if element is None:
            raise ElementNotExistError(locator=locator)
        LOGGER.info('Type text("{}") into element("{}").'.format(text, locator))
        if clear:
            value = await element.executionContext.evaluate(
                "(node => node.value || node.innerText)", element
            )
            if value:
                await _clickable(element)
                await element.click()
                for _ in value:
                    await self.pyppeteer_page.keyboard.press("ArrowRight")
                await self.pyppeteer_page.keyboard.down("Shift")
                for _ in value:
                    await self.pyppeteer_page.keyboard.press("ArrowLeft")
                    await asyncio.sleep(delay / 1000.0)
                await self.pyppeteer_page.keyboard.up("Shift")
                await self.pyppeteer_page.keyboard.press("Backspace")
        await element.type(text, delay=delay)
        await element.dispose()

    async def click(
        self,
        locator: str,
        button: str = "left",
        click_count: int = 1,
        delay: int = 0,
    ):
        """Click the center of the element which matches ``locator``.

        :param str locator: a selector or xpath string.
        :param str button: ``left``, ``right``, of ``middle``.
               Defaults to ``left``.
        :param int click_count: Defaults to 1.
        :param int delay: Time to wait between ``mousedown`` and
               ``mouseup`` in milliseconds. Defaults to 0.
        :return: None
        :raise ElementNotExistError: if the element which matches
               ``locator`` is not found.
        """
        element = await self.query_locator(locator)
        if element is None:
            raise ElementNotExistError(locator=locator)
        LOGGER.info('Click element("{}").'.format(locator))
        await _clickable(element)
        await element.click(button=button, clickCount=click_count, delay=delay)
        await element.dispose()

    async def get_value(self, locator: str) -> str:
        """Get the element ``value`` or ``innerText`` which matches
        ``locator``.

        :param str locator: a selector or xpath string.
        :return: the element ``value`` or ``innerText`` string.
        :raise ElementNotExistError: if the element which matches
               ``locator`` is not found.
        """
        element = await self.query_locator(locator)
        if element is None:
            raise ElementNotExistError(locator=locator)
        value: str = await element.executionContext.evaluate(
            "(node => node.value || node.innerText)", element
        )
        await element.dispose()
        return value



from pathlib import Path

from setuptools import find_packages, setup

with Path("README.md").open(encoding="utf-8") as readme:
    long_description = readme.read()

setup(
    name="pytest-pyppeteer",
    version="0.2.2",
    author="Yao Meng",
    author_email="yaomeng614@gmail.com",
    maintainer="Yao Meng",
    maintainer_email="yaomeng614@gmail.com",
    description="A plugin to run pyppeteer in pytest.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="pytest-plugin pyppeteer puppeteer",
    url="https://github.com/luizyao/pytest-pyppeteer",
    packages=find_packages("src"),
    package_dir={"": "src"},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Testing",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "License :: OSI Approved :: MIT License",
    ],
    entry_points={"pytest11": ["pytest_pyppeteer = pytest_pyppeteer.plugin"]},
    python_requires=">=3.7",
    install_requires=[
        "pyppeteer==0.2.2",
        "pytest-asyncio>=0.14.0",
        "pytest>=6.0.2",
        "pydantic>=1.6.1",
        "cssselect>=1.1.0",
        "lxml>=4.5.2",
    ],
)
